@@ -53,6 +53,9 @@
 
 #define GLADE_CONFIG_FILENAME "glade.conf"
 
+#define GLADE_APP_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+                                       GLADE_TYPE_APP,                        \
+                                       GladeAppPrivate))
 enum
 {
   DOC_SEARCH,
@@ -86,8 +89,6 @@ static gchar *bin_dir = NULL;
 static GladeApp *singleton_app = NULL;
 static gboolean check_initialised = FALSE;
 
-static void glade_init_check (void);
-
 G_DEFINE_TYPE (GladeApp, glade_app, G_TYPE_OBJECT);
 
 /*****************************************************************
@@ -154,66 +155,6 @@ glade_app_finalize (GObject * app)
   G_OBJECT_CLASS (glade_app_parent_class)->finalize (app);
 }
 
-/*****************************************************************
- *                    GladeAppClass                              *
- *****************************************************************/
-static GKeyFile *
-glade_app_config_load (GladeApp * app)
-{
-  GKeyFile *config = g_key_file_new ();
-  gchar *filename;
-
-  filename =
-      g_build_filename (g_get_user_config_dir (), GLADE_CONFIG_FILENAME, NULL);
-
-  g_key_file_load_from_file (config, filename, G_KEY_FILE_NONE, NULL);
-
-  g_free (filename);
-
-  return config;
-}
-
-const gchar *
-glade_app_get_catalogs_dir (void)
-{
-  glade_init_check ();
-
-  return catalogs_dir;
-}
-
-const gchar *
-glade_app_get_modules_dir (void)
-{
-  glade_init_check ();
-
-  return modules_dir;
-}
-
-const gchar *
-glade_app_get_pixmaps_dir (void)
-{
-  glade_init_check ();
-
-  return pixmaps_dir;
-}
-
-const gchar *
-glade_app_get_locale_dir (void)
-{
-  glade_init_check ();
-
-  return locale_dir;
-}
-
-const gchar *
-glade_app_get_bin_dir (void)
-{
-  glade_init_check ();
-
-  return bin_dir;
-}
-
-
 /* build package paths at runtime */
 static void
 build_package_paths (void)
@@ -263,17 +204,117 @@ glade_init_check (void)
   check_initialised = TRUE;
 }
 
+/*****************************************************************
+ *                    GladeAppClass                              *
+ *****************************************************************/
+const gchar *
+glade_app_get_catalogs_dir (void)
+{
+  glade_init_check ();
+
+  return catalogs_dir;
+}
+
+const gchar *
+glade_app_get_modules_dir (void)
+{
+  glade_init_check ();
+
+  return modules_dir;
+}
+
+const gchar *
+glade_app_get_pixmaps_dir (void)
+{
+  glade_init_check ();
+
+  return pixmaps_dir;
+}
+
+const gchar *
+glade_app_get_locale_dir (void)
+{
+  glade_init_check ();
+
+  return locale_dir;
+}
+
+const gchar *
+glade_app_get_bin_dir (void)
+{
+  glade_init_check ();
+
+  return bin_dir;
+}
+
 static void
-glade_app_init (GladeApp * app)
+pointer_mode_register_icon (GtkIconFactory *factory,
+                            const gchar *icon_name,
+                            GladePointerMode mode,
+                            GtkIconSize size)
+{
+  GdkPixbuf *pixbuf;
+
+  pixbuf = glade_utils_pointer_mode_render_icon (mode, size);
+  if (pixbuf)
+    {
+      GtkIconSet *icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
+      gtk_icon_factory_add (factory, icon_name, icon_set);
+      g_object_unref (pixbuf);
+    }
+}
+
+/*
+ * glade_app_register_stock_icons:
+ * @size: icon size
+ *
+ * Register a new stock icon for most of GladePointerMode.
+ * After calling this function "glade-selector", "glade-drag-resize",
+ * "glade-margin-edit" and "glade-align-edit" stock icons will be available.
+ */ 
+static void
+glade_app_register_stock_icons (GtkIconSize size)
+{
+  GtkIconFactory *factory = gtk_icon_factory_new ();
+
+  pointer_mode_register_icon (factory, "glade-selector", GLADE_POINTER_SELECT, size);
+  pointer_mode_register_icon (factory, "glade-drag-resize", GLADE_POINTER_DRAG_RESIZE, size);
+  pointer_mode_register_icon (factory, "glade-margin-edit", GLADE_POINTER_MARGIN_EDIT, size);
+  pointer_mode_register_icon (factory, "glade-align-edit", GLADE_POINTER_ALIGN_EDIT, size);
+
+  gtk_icon_factory_add_default (factory);
+}
+
+/**
+ * glade_init:
+ * 
+ * Initialization function for libgladeui (not #GladeApp)
+ * It builds paths, bind text domain, and register icons
+ */
+void
+glade_init (void)
+{
+  static gboolean init = FALSE;
+
+  if (init) return;
+  
+  glade_init_check ();
+
+  /* Register icons needed by the UI */
+  glade_app_register_stock_icons (GTK_ICON_SIZE_LARGE_TOOLBAR);
+
+  init = TRUE;
+}
+
+static void
+glade_app_init (GladeApp *app)
 {
   static gboolean initialized = FALSE;
-  GladeAppPrivate *priv =
-    GLADE_APP (app)->priv = 
-    G_TYPE_INSTANCE_GET_PRIVATE ((app), GLADE_TYPE_APP, GladeAppPrivate);
+  GladeAppPrivate *priv = app->priv = GLADE_APP_GET_PRIVATE (app);
 
   singleton_app = app;
 
-  glade_init_check ();
+  glade_init ();
 
   if (!initialized)
     {
@@ -294,7 +335,7 @@ glade_app_init (GladeApp * app)
   priv->clipboard = glade_clipboard_new ();
 
   /* Load the configuration file */
-  priv->config = glade_app_config_load (app);
+  priv->config = g_key_file_ref (glade_app_get_config ());
 }
 
 static void
@@ -598,8 +639,18 @@ glade_app_get_projects (void)
 GKeyFile *
 glade_app_get_config (void)
 {
-  GladeApp *app = glade_app_get ();
-  return app->priv->config;
+  static GKeyFile *config = NULL;
+
+  if (config == NULL)
+    {
+      gchar *filename = g_build_filename (g_get_user_config_dir (),
+                                          GLADE_CONFIG_FILENAME, NULL);
+      config = g_key_file_new ();
+      g_key_file_load_from_file (config, filename, G_KEY_FILE_NONE, NULL);
+      g_free (filename);
+    }
+
+  return config;
 }
 
 gboolean
