@@ -183,6 +183,14 @@ extern void _glade_marshal_INT__OBJECT_BOXED (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
+/* BOXED:OBJECT (./glade-marshallers.list:25) */
+extern void _glade_marshal_BOXED__OBJECT (GClosure     *closure,
+                                          GValue       *return_value,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint,
+                                          gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* ___glade_marshal_MARSHAL_H__ */
