@@ -855,7 +855,7 @@ try_load_library (const gchar *library_path, const gchar *library_name)
   gchar *path;
 
   path = g_module_build_path (library_path, library_name);
-  if (!library_path || g_file_test (path, G_FILE_TEST_EXISTS))
+  if (g_file_test (path, G_FILE_TEST_EXISTS))
     {
       if (!(module = g_module_open (path, G_MODULE_BIND_LAZY)))
         g_warning ("Failed to load %s: %s", path, g_module_error ());
@@ -882,15 +882,19 @@ glade_util_load_library (const gchar *library_name)
 {
   gchar *default_paths[] = { (gchar *) glade_app_get_modules_dir (),
     NULL,                       /* <-- dynamically allocated */
+    "/lib",
+    "/usr/lib",
     "/usr/local/lib",
     NULL
   };
-  
+
   GModule *module = NULL;
   const gchar *search_path;
   gchar **split;
   gint i;
 
+
+
   if ((search_path = g_getenv (GLADE_ENV_MODULE_PATH)) != NULL)
     {
       if ((split = g_strsplit (search_path, ":", 0)) != NULL)
@@ -909,7 +913,7 @@ glade_util_load_library (const gchar *library_name)
       default_paths[1] =
           g_build_filename (glade_app_get_modules_dir (), "..", "..", NULL);
 
-      for (i = 0; i < G_N_ELEMENTS (default_paths); i++)
+      for (i = 0; default_paths[i] != NULL; i++)
         if ((module =
              try_load_library (default_paths[i], library_name)) != NULL)
           break;
@@ -1741,3 +1745,190 @@ glade_utils_replace_home_dir_with_tilde (const gchar *path)
   return g_strdup (path);
 #endif
 }
+
+static void
+draw_tip (cairo_t *cr)
+{
+  cairo_line_to (cr, 2, 8);
+  cairo_line_to (cr, 2, 4);
+  cairo_line_to (cr, 0, 4);
+  cairo_line_to (cr, 0, 3);
+  cairo_line_to (cr, 3, 0);
+  cairo_line_to (cr, 6, 3);
+  cairo_line_to (cr, 6, 4);
+  cairo_line_to (cr, 4, 4);
+
+  cairo_translate (cr, 12, 6);
+  cairo_rotate (cr, G_PI_2);
+}
+
+static void
+draw_tips (cairo_t *cr)
+{
+  cairo_move_to (cr, 2, 8);
+  draw_tip (cr); draw_tip (cr); draw_tip (cr); draw_tip (cr);
+  cairo_close_path (cr);
+}
+
+static void
+draw_pointer (cairo_t *cr)
+{
+  cairo_line_to (cr, 8, 3);
+  cairo_line_to (cr, 19, 14);
+  cairo_line_to (cr, 13.75, 14);
+  cairo_line_to (cr, 16.5, 19);
+  cairo_line_to (cr, 14, 21);
+  cairo_line_to (cr, 11, 16);
+  cairo_line_to (cr, 7, 19);
+  cairo_line_to (cr, 7, 3);
+  cairo_line_to (cr, 8, 3);
+}
+
+/* Needed for private draw functions! */
+#include "glade-design-private.h"
+
+/**
+ * glade_utils_pointer_mode_render_icon:
+ * @mode: the #GladePointerMode to render as icon
+ * @size: icon size
+ *
+ * Render an icon representing the pointer mode.
+ * Best view with sizes bigger than GTK_ICON_SIZE_LARGE_TOOLBAR.
+ */ 
+GdkPixbuf *
+glade_utils_pointer_mode_render_icon (GladePointerMode mode, GtkIconSize size)
+{
+  GtkStyleContext *ctx = gtk_style_context_new ();
+  GdkRGBA c1, c2, c3, fg, bg;
+  cairo_surface_t *surface;
+  GtkWidgetPath *path;
+  gint width, height;
+  GdkPixbuf *pix;
+  cairo_t *cr;
+
+  if (gtk_icon_size_lookup (size, &width, &height) == FALSE) return NULL;
+
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
+  cr = cairo_create (surface);
+  cairo_scale (cr, width/24.0, height/24.0);
+
+  /* Get Style context */
+  path = gtk_widget_path_new ();
+  gtk_widget_path_append_type (path, GTK_TYPE_WIDGET);
+  gtk_style_context_set_path (ctx, path);
+  gtk_widget_path_free (path);
+
+  /* Now get colors */
+  gtk_style_context_lookup_color (ctx, "fg_color", &fg);
+  gtk_style_context_lookup_color (ctx, "bg_color", &bg);
+  gtk_style_context_lookup_color (ctx, "selected_bg_color", &c1);
+  gtk_style_context_lookup_color (ctx, "selected_fg_color", &c2);
+  gtk_style_context_lookup_color (ctx, "base_color", &c3);
+  
+  g_object_unref (ctx);
+
+  /* Clear surface */
+  cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+  cairo_fill(cr);
+  cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+
+  switch (mode)
+    {
+      case GLADE_POINTER_SELECT:
+      case GLADE_POINTER_ADD_WIDGET:
+        cairo_set_line_width (cr, 1);
+        cairo_translate (cr, 1.5, 1.5);
+        draw_pointer (cr);
+        fg.alpha = .16;
+        gdk_cairo_set_source_rgba (cr, &fg);
+        cairo_stroke (cr);
+
+        cairo_translate (cr, -1, -1);
+        draw_pointer (cr);
+        gdk_cairo_set_source_rgba (cr, &c2);
+        cairo_fill_preserve (cr);
+      
+        fg.alpha = .64;
+        gdk_cairo_set_source_rgba (cr, &fg);
+        cairo_stroke (cr);
+      break;
+      case GLADE_POINTER_DRAG_RESIZE:
+        cairo_set_line_width (cr, 1);
+        cairo_translate (cr, 10.5, 3.5);
+        
+        draw_tips (cr);
+
+        fg.alpha = .16;
+        gdk_cairo_set_source_rgba (cr, &fg);
+        cairo_stroke (cr);
+
+        cairo_translate (cr, -1, -1);
+        draw_tips (cr);
+        
+        gdk_cairo_set_source_rgba (cr, &c2);
+        cairo_fill_preserve (cr);
+        
+        c1.red = MAX (0, c1.red - .1);
+        c1.green = MAX (0, c1.green - .1);
+        c1.blue = MAX (0, c1.blue - .1);
+        gdk_cairo_set_source_rgba (cr, &c1);
+        cairo_stroke (cr);
+      break;
+      case GLADE_POINTER_MARGIN_EDIT:
+        {
+          gdk_cairo_set_source_rgba (cr, &bg);
+          cairo_rectangle (cr, 4, 4, 18, 18);
+          cairo_fill (cr);
+
+          c1.alpha = .1;
+          gdk_cairo_set_source_rgba (cr, &c1);
+          cairo_rectangle (cr, 6, 6, 16, 16);
+          cairo_fill (cr);
+
+          cairo_set_line_width (cr, 1);
+          fg.alpha = .32;
+          gdk_cairo_set_source_rgba (cr, &fg);
+          cairo_move_to (cr, 16.5, 22);
+          cairo_line_to (cr, 16.5, 16.5);
+          cairo_line_to (cr, 22, 16.5);
+          cairo_stroke (cr);
+
+          c1.alpha = .16;
+          gdk_cairo_set_source_rgba (cr, &c1);
+          cairo_rectangle (cr, 16, 16, 6, 6);
+          cairo_fill (cr);
+
+          cairo_set_line_width (cr, 2);
+          c1.alpha = .75;
+          gdk_cairo_set_source_rgba (cr, &c1);
+          cairo_move_to (cr, 6, 22);
+          cairo_line_to (cr, 6, 6);
+          cairo_line_to (cr, 22, 6);
+          cairo_stroke (cr);
+
+          c1.alpha = 1;
+          cairo_scale (cr, .75, .75);
+          cairo_set_line_width (cr, 4);
+          _glade_design_layout_draw_node (cr, 16*1.25, 6*1.25, &c1, &c2);
+          _glade_design_layout_draw_node (cr, 6*1.25, 16*1.25, &c1, &c2);
+        }
+      break;
+      case GLADE_POINTER_ALIGN_EDIT:
+        cairo_scale (cr, 1.5, 1.5);
+        cairo_rotate (cr, 45*(G_PI/180));
+        cairo_translate (cr, 11, 2);
+        _glade_design_layout_draw_pushpin (cr, 2.5, &c1, &c2, &c2, &fg);
+      break;
+      default:
+      break;
+    }
+
+  pix = gdk_pixbuf_get_from_surface (surface, 0, 0,
+                                     cairo_image_surface_get_width (surface),
+                                     cairo_image_surface_get_height (surface));
+
+  cairo_surface_destroy (surface);
+  cairo_destroy (cr);
+
+  return pix;
+}
